package plan9

import Chisel._
import util.Random

abstract trait CarpParameters {
  var TxBufferAddressBits   : Int     = 8
  var RxBufferAddressBits   : Int     = 8
  var ReversePayloadEndian  : Boolean = true
  var ProgramCounterBits    : Int     = 8
  var MatrixWidth           : Int     = 8
  var MatrixHeight          : Int     = 8
  var MatrixDepth           : Int     = 1
  var MatrixWrap            : Boolean = true
  var CellTypeBits          : Int     = 5
  var CellStateBits         : Int     = 1
  var JumpCounters          : Int     = 4
  var JumpCounterBits       : Int     = 16
  var InstructionBits       : Int     = 256
  var LUTConfigurationBits  : Int     = 2
  var RuleAmount            : Int     = 256
  var RulesTestedInParallel : Int     = 2
  var RuleVectorAmount      : Int     = 64
  var LiveCountBufferSize   : Int     = 256
  var FitnessBufferSize     : Int     = 256
  var ReadoutBufferSize     : Int     = 256

  def coordToIndex(c: Array[Int]): Int = c(0) * MatrixHeight * MatrixWidth + c(1) * MatrixWidth + c(2)
  def coordToIndex(z: Int, y: Int, x: Int): Int = z * MatrixHeight * MatrixWidth + y * MatrixWidth + x
  // (z, y, x)
  var CAOutputCells: Array[Array[Int]] = Array(
    Array(0,0,0), Array(0,0,1), Array(0,0,2),
    Array(0,1,0),               Array(0,1,2),
    Array(0,2,0), Array(0,2,1), Array(0,2,2)
  )
  // (z, y, x)
  var CAFeedbackCells : Array[Array[Int]] = Array(Array(0,1,1), Array(0,1,5), Array(0,5,1), Array(0,5,5))
  var SNNTopology: Array[Int] = Array(1)
  var SNNWeightWidth: Int = 8
  var SNNWeightAddrWidth: Int = 16
  def SNNWeightCount(): Int = Array.tabulate(SNNTopology.size){i =>
    SNNTopology(i)*(if (i == 0) CAOutputCells.size else SNNTopology(i-1))
  }.foldLeft(0)(_ + _)

  var CellTypeWriteWidth =
    math.min(MatrixWidth, (InstructionBits - 32) / CellTypeBits)
  var CellStateWriteWidth =
    math.min(MatrixWidth, (InstructionBits - 32) / CellTypeBits)
  var LiveCountBufferBits =
    log2Ceil(MatrixDepth * MatrixHeight * MatrixWidth)
  var SendBufferSize = math.pow(2, TxBufferAddressBits.toDouble).ceil.toInt
  var NeighborhoodSize = if (MatrixDepth == 1) 5 else 7
}

object DefaultParams extends CarpParameters
