package plan9

import Chisel._

class CellStorage(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle {
    val aAdressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    aAdressZ.setName("a_address_z")
    val aAdressY = Bits(INPUT, width=log2Up(cp.MatrixHeight))
    aAdressY.setName("a_address_y")
    val aTypesWrite = Bool(INPUT)
    aTypesWrite.setName("a_types_write")
    val aTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    aTypesIn.setName("a_types_in")
    val aTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    aTypesOut.setName("a_types_out")
    val aStatesWrite = Bool(INPUT)
    aStatesWrite.setName("a_states_write")
    val aStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    aStatesIn.setName("a_states_in")
    val aStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    aStatesOut.setName("a_states_out")

    val bAdressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    bAdressZ.setName("b_address_z")
    val bAdressY = Bits(INPUT, width=log2Up(cp.MatrixHeight))
    bAdressY.setName("b_address_y")
    val bTypesWrite = Bool(INPUT)
    bTypesWrite.setName("b_types_write")
    val bTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bTypesIn.setName("b_types_in")
    val bTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bTypesOut.setName("b_types_out")
    val bStatesWrite = Bool(INPUT)
    bStatesWrite.setName("b_states_write")
    val bStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bStatesIn.setName("b_states_in")
    val bStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bStatesOut.setName("b_states_out")

    val operation = UInt(INPUT, width=log2Up(Types.cellStorageOperation.size))
    operation.setName("operation")

    val run = Bool(INPUT)
    run.setName("run")
  }
  setVerilogParameters(new VerilogParameters {
    val matrix_width = cp.MatrixWidth
    val matrix_height = cp.MatrixHeight
    val matrix_depth = cp.MatrixDepth
    val cell_type_bits = cp.CellTypeBits
    val cell_state_bits = cp.CellStateBits
  })
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
