package plan9

import Chisel._

class CellStorageMux(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle {
    // Cell Writer Reader
    val writerReaderAddressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    writerReaderAddressZ.setName("writer_reader_address_z")
    val writerReaderAddressY = Bits(INPUT, width=log2Up(cp.MatrixHeight))
    writerReaderAddressY.setName("writer_reader_address_y")
    val writerReaderTypesWrite = Bool(INPUT)
    writerReaderTypesWrite.setName("writer_reader_types_write")
    val writerReaderTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    writerReaderTypesIn.setName("writer_reader_types_in")
    val writerReaderTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    writerReaderTypesOut.setName("writer_reader_types_out")
    val writerReaderStatesWrite = Bool(INPUT)
    writerReaderStatesWrite.setName("writer_reader_states_write")
    val writerReaderStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    writerReaderStatesIn.setName("writer_reader_states_in")
    val writerReaderStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    writerReaderStatesOut.setName("writer_reader_states_out")

    // Cellular Automata
    val cellularAutomataAddressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    cellularAutomataAddressZ.setName("cellular_automata_address_z")
    val cellularAutomataAddressY = Bits(INPUT, width=log2Up(cp.MatrixHeight))
    cellularAutomataAddressY.setName("cellular_automata_address_y")
    val cellularAutomataTypesWrite = Bool(INPUT)
    cellularAutomataTypesWrite.setName("cellular_automata_types_write")
    val cellularAutomataTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    cellularAutomataTypesIn.setName("cellular_automata_types_in")
    val cellularAutomataTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    cellularAutomataTypesOut.setName("cellular_automata_types_out")
    val cellularAutomataStatesWrite = Bool(INPUT)
    cellularAutomataStatesWrite.setName("cellular_automata_states_write")
    val cellularAutomataStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    cellularAutomataStatesIn.setName("cellular_automata_states_in")
    val cellularAutomataStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    cellularAutomataStatesOut.setName("cellular_automata_states_out")

    // Development - Port A
    val developmentAAddressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    developmentAAddressZ.setName("development_a_address_z")
    val developmentAAddressY = Bits(INPUT, width=(log2Up(cp.MatrixHeight)))
    developmentAAddressY.setName("development_a_address_y")
    val developmentATypesWrite = Bool(INPUT)
    developmentATypesWrite.setName("development_a_types_write")
    val developmentATypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    developmentATypesIn.setName("development_a_types_in")
    val developmentATypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    developmentATypesOut.setName("development_a_types_out")
    val developmentAStatesWrite = Bool(INPUT)
    developmentAStatesWrite.setName("development_a_states_write")
    val developmentAStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    developmentAStatesIn.setName("development_a_states_in")
    val developmentAStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    developmentAStatesOut.setName("development_a_states_out")

    // Development - Port A
    val developmentBAddressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    developmentBAddressZ.setName("development_b_address_z")
    val developmentBAddressY = Bits(INPUT, width=(log2Up(cp.MatrixHeight)))
    developmentBAddressY.setName("development_b_address_y")
    val developmentBTypesWrite = Bool(INPUT)
    developmentBTypesWrite.setName("development_b_types_write")
    val developmentBTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    developmentBTypesIn.setName("development_b_types_in")
    val developmentBTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    developmentBTypesOut.setName("development_b_types_out")
    val developmentBStatesWrite = Bool(INPUT)
    developmentBStatesWrite.setName("development_b_states_write")
    val developmentBStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    developmentBStatesIn.setName("development_b_states_in")
    val developmentBStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    developmentBStatesOut.setName("development_b_states_out")

    // BRAM - Port A
    val bramAAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    bramAAddressZ.setName("bram_a_address_z")
    val bramAAddressY = Bits(OUTPUT, width=(log2Up(cp.MatrixHeight)))
    bramAAddressY.setName("bram_a_address_y")
    val bramATypesWrite = Bool(OUTPUT)
    bramATypesWrite.setName("bram_a_types_write")
    val bramATypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramATypesIn.setName("bram_a_types_in")
    val bramATypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramATypesOut.setName("bram_a_types_out")
    val bramAStatesWrite = Bool(OUTPUT)
    bramAStatesWrite.setName("bram_a_states_write")
    val bramAStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramAStatesIn.setName("bram_a_states_in")
    val bramAStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramAStatesOut.setName("bram_a_states_out")

    // BRAM - Port B
    val bramBAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    bramBAddressZ.setName("bram_b_address_z")
    val bramBAddressY = Bits(OUTPUT, width=(log2Up(cp.MatrixHeight)))
    bramBAddressY.setName("bram_b_address_y")
    val bramBTypesWrite = Bool(OUTPUT)
    bramBTypesWrite.setName("bram_b_types_write")
    val bramBTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramBTypesIn.setName("bram_b_types_in")
    val bramBTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramBTypesOut.setName("bram_b_types_out")
    val bramBStatesWrite = Bool(OUTPUT)
    bramBStatesWrite.setName("bram_b_states_write")
    val bramBStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramBStatesIn.setName("bram_b_states_in")
    val bramBStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramBStatesOut.setName("bram_b_states_out")

    val sourceSelect = UInt(INPUT, width=log2Up(Types.cellStorageMuxSelect.size))
    sourceSelect.setName("source_select")
    val run = Bool(INPUT)
    run.setName("run")
  }

  setVerilogParameters(new VerilogParameters {
    val matrix_width = cp.MatrixWidth
    val matrix_height = cp.MatrixHeight
    val matrix_depth = cp.MatrixDepth
    val cell_type_bits = cp.CellTypeBits
    val cell_state_bits = cp.CellStateBits
  })
  setName("cell_storage_mux")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
