package plan9

import Chisel._

class CellWriterReader(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle {
    val bufferAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    bufferAddressZ.setName("buffer_address_z")
    val bufferAddressY = Bits(OUTPUT, width=log2Up(cp.MatrixHeight))
    bufferAddressY.setName("buffer_address_y")
    val bufferTypesWrite = Bool(OUTPUT)
    bufferTypesWrite.setName("buffer_types_write")
    val bufferTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bufferTypesOut.setName("buffer_types_out")
    val bufferTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bufferTypesIn.setName("buffer_types_in")
    val bufferStatesWrite = Bool(OUTPUT)
    bufferStatesWrite.setName("buffer_states_write")
    val bufferStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bufferStatesOut.setName("buffer_states_out")
    val bufferStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bufferStatesIn.setName("buffer_states_in")

    val decodeOperation = UInt(INPUT, width=log2Up(Types.cellWriterReaderOperation.size))
    decodeOperation.setName("decode_operation")
    val decodeAddressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    decodeAddressZ.setName("decode_address_z")
    val decodeAddressY = Bits(INPUT, width=log2Up(cp.MatrixHeight))
    decodeAddressY.setName("decode_address_y")
    val decodeAddressX = Bits(INPUT, width=log2Up(cp.MatrixWidth))
    decodeAddressX.setName("decode_address_x")
    val decodeState = Bits(INPUT, width=cp.CellStateBits)
    decodeState.setName("decode_state")
    val decodeStates = Bits(INPUT, width=cp.CellStateWriteWidth*cp.CellStateBits)
    decodeStates.setName("decode_states")
    val decodeType = Bits(INPUT, width=cp.CellTypeBits)
    decodeType.setName("decode_type")
    val decodeTypes = Bits(INPUT, width=cp.CellTypeWriteWidth*cp.CellTypeBits)
    decodeTypes.setName("decode_types")

    val sendBufferCount = Bits(INPUT, width=cp.TxBufferAddressBits)
    sendBufferCount.setName("send_buffer_count")
    val sendBufferData = Bits(OUTPUT, width=32)
    sendBufferData.setName("send_buffer_data")
    val sendBufferWrite = Bool(OUTPUT)
    sendBufferWrite.setName("send_buffer_write")

    val run = Bool(INPUT)
    run.setName("run")
    val done = Bool(OUTPUT)
    done.setName("done")
  }

  setVerilogParameters(new VerilogParameters {
    val matrix_width = cp.MatrixWidth
    val matrix_height = cp.MatrixHeight
    val matrix_depth = cp.MatrixDepth
    val cell_type_bits = cp.CellTypeBits
    val cell_state_bits = cp.CellStateBits
    val cell_type_write_width = cp.CellTypeWriteWidth
    val cell_state_write_width = cp.CellStateWriteWidth
    val send_buffer_address_bits = cp.TxBufferAddressBits
  })
  setName("cell_writer_reader")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
