package plan9

import Chisel._

class Decode(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val ruleWriterDataWidth =
    (cp.CellTypeBits + 1 + cp.CellStateBits + 1) * (cp.NeighborhoodSize + 1)

  val io = new Bundle {
    val instruction = Bits(INPUT, width=cp.InstructionBits)
    instruction.setName("instruction")

    val run = Bool(INPUT)
    run.setName("run")

    val informationSenderOperation = UInt(OUTPUT, width=log2Up(Types.informationSenderOperation.size))
    informationSenderOperation.setName("information_sender_operation")

    val cellWriterReaderOperation = UInt(OUTPUT, width=log2Up(Types.cellWriterReaderOperation.size))
    cellWriterReaderOperation.setName("cell_writer_reader_operation")
    val cellWriterReaderAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    cellWriterReaderAddressZ.setName("cell_writer_reader_address_z")
    val cellWriterReaderAddressY = Bits(OUTPUT, width=log2Up(cp.MatrixHeight))
    cellWriterReaderAddressY.setName("cell_writer_reader_address_y")
    val cellWriterReaderAddressX = Bits(OUTPUT, width=log2Up(cp.MatrixWidth))
    cellWriterReaderAddressX.setName("cell_writer_reader_address_x")
    val cellWriterReaderState = Bits(OUTPUT, width=cp.CellStateBits)
    cellWriterReaderState.setName("cell_writer_reader_state")
    val cellWriterReaderStates = Bits(OUTPUT, width=cp.CellStateWriteWidth*cp.CellStateBits)
    cellWriterReaderStates.setName("cell_writer_reader_states")
    val cellWriterReaderType = Bits(OUTPUT, width=cp.CellTypeBits)
    cellWriterReaderType.setName("cell_writer_reader_type")
    val cellWriterReaderTypes = Bits(OUTPUT, width=cp.CellTypeWriteWidth*cp.CellTypeBits)
    cellWriterReaderTypes.setName("cell_writer_reader_types")

    val cellStorageMuxSelect = UInt(OUTPUT, width=log2Up(Types.cellStorageMuxSelect.size))
    cellStorageMuxSelect.setName("cell_storage_mux_select")

    val sendBufferMuxSelect = UInt(OUTPUT, width=log2Up(Types.sendBufferMuxSelect.size))
    sendBufferMuxSelect.setName("send_buffer_mux_select")

    val ruleWriterOperation = UInt(OUTPUT, width=log2Up(Types.ruleWriterOperation.size))
    ruleWriterOperation.setName("rule_writer_operation")
    val ruleWriterAddress = Bits(OUTPUT, width=log2Up(cp.RuleAmount))
    ruleWriterAddress.setName("rule_writer_address")
    val ruleWriterData = Bits(OUTPUT, width=ruleWriterDataWidth)
    ruleWriterData.setName("rule_writer_data")

    val developmentOperation = UInt(OUTPUT, width=log2Up(Types.developmentOperation.size))
    developmentOperation.setName("development_operation")
    val developmentRulesActive = Bits(OUTPUT, width=log2Up(cp.RuleAmount))
    developmentRulesActive.setName("development_rules_active")

    val ruleNumbersReaderOperation = UInt(OUTPUT, width=log2Up(Types.ruleNumbersReaderOperation.size))
    ruleNumbersReaderOperation.setName("rule_numbers_reader_operation")

    val ruleVectorReaderOperation = UInt(OUTPUT, width=log2Up(Types.ruleVectorReaderOperation.size))
    ruleVectorReaderOperation.setName("rule_vector_reader_operation")
    val ruleVectorReaderCount = Bits(OUTPUT, width=log2Up(cp.RuleVectorAmount))
    ruleVectorReaderCount.setName("rule_vector_reader_count")

    val lutWriterOperation = UInt(OUTPUT, width=log2Up(Types.lutWriterOperation.size))
    lutWriterOperation.setName("lut_writer_operation")
    val lutWriterAddress = Bits(OUTPUT, width=cp.CellTypeBits)
    lutWriterAddress.setName("lut_writer_address")
    val lutWriterData = Bits(OUTPUT, width=Math.pow(2, (cp.NeighborhoodSize).toDouble).ceil.toInt)
    lutWriterData.setName("lut_writer_data")

    val fitnessSenderOperation = UInt(OUTPUT, width=log2Up(Types.fitnessSenderOperation.size))
    fitnessSenderOperation.setName("fitness_sender_operation")

    val readoutSenderOperation = UInt(OUTPUT, width=log2Up(Types.readoutSenderOperation.size))
    readoutSenderOperation.setName("readout_sender_operation")

    val cellularAutomataOperation = UInt(OUTPUT, width=log2Up(Types.cellularAutomataOperation.size))
    cellularAutomataOperation.setName("cellular_automata_operation")
    val cellularAutomataStepCount = UInt(OUTPUT, width=16) // Parameterize?
    cellularAutomataStepCount.setName("cellular_automata_step_count")

    val cellStorageOperation = UInt(OUTPUT, width=log2Up(Types.cellStorageOperation.size))
    cellStorageOperation.setName("cell_storage_operation")

    val resetterOperation = UInt(OUTPUT, width=log2Up(Types.resetterOperation.size))
    resetterOperation.setName("resetter_operation")

    val readoutOperation = UInt(OUTPUT, width=log2Up(Types.readoutOperation.size))
    readoutOperation.setName("readout_operation")
    val readoutWeightAddress = Bits(OUTPUT, width=log2Up(cp.SNNWeightCount()))
    readoutWeightAddress.setName("readout_weight_address")
    val readoutWeightData = Bits(OUTPUT, width=cp.SNNWeightWidth)
    readoutWeightData.setName("readout_weight_data")
  }

  setVerilogParameters(new VerilogParameters {
    val matrix_width = cp.MatrixWidth
    val matrix_height = cp.MatrixHeight
    val matrix_depth = cp.MatrixDepth
    val cell_type_bits = cp.CellTypeBits
    val cell_state_bits = cp.CellStateBits
    val cell_type_write_width = cp.CellTypeWriteWidth
    val cell_state_write_width = cp.CellStateWriteWidth
    val instruction_bits = cp.InstructionBits
    val rule_amount = cp.RuleAmount
    val rule_vector_amount = cp.RuleVectorAmount
    val snn_weight_addr_weight = cp.SNNWeightAddrWidth
    val snn_weight_width = cp.SNNWeightWidth
  })
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
