package plan9

import Chisel._

class Development(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams
  val ruleWidth = (cp.CellTypeBits + 1 + cp.CellStateBits + 1) * (cp.NeighborhoodSize + 1)

  val io = new Bundle {
    // Buffer - Port A
    val bramAAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    bramAAddressZ.setName("bram_a_address_z")
    val bramAAddressY = Bits(OUTPUT, width=log2Up(cp.MatrixHeight))
    bramAAddressY.setName("bram_a_address_y")
    val bramATypesWrite = Bool(OUTPUT)
    bramATypesWrite.setName("bram_a_types_write")
    val bramATypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramATypesIn.setName("bram_a_types_in")
    val bramATypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramATypesOut.setName("bram_a_types_out")
    val bramAStatesWrite = Bool(OUTPUT)
    bramAStatesWrite.setName("bram_a_states_write")
    val bramAStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramAStatesIn.setName("bram_a_states_in")
    val bramAStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramAStatesOut.setName("bram_a_states_out")

    // Buffer - Port B
    val bramBAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    bramBAddressZ.setName("bram_b_address_z")
    val bramBAddressY = Bits(OUTPUT, width=log2Up(cp.MatrixHeight))
    bramBAddressY.setName("bram_b_address_y")
    val bramBTypesWrite = Bool(OUTPUT)
    bramBTypesWrite.setName("bram_b_types_write")
    val bramBTypesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramBTypesIn.setName("bram_b_types_in")
    val bramBTypesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellTypeBits)
    bramBTypesOut.setName("bram_b_types_out")
    val bramBStatesWrite = Bool(OUTPUT)
    bramBStatesWrite.setName("bram_b_states_write")
    val bramBStatesIn = Bits(INPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramBStatesIn.setName("bram_b_states_in")
    val bramBStatesOut = Bits(OUTPUT, width=cp.MatrixWidth*cp.CellStateBits)
    bramBStatesOut.setName("bram_b_states_out")

    // Rule storage
    val ruleStorageWrite = Bool(INPUT)
    ruleStorageWrite.setName("rule_storage_write")
    val ruleStorageAddress = Bits(INPUT, width=log2Up(cp.RuleAmount))
    ruleStorageAddress.setName("rule_storage_address")
    val ruleStorageData = Bits(INPUT, width=ruleWidth)
    ruleStorageData.setName("rule_storage_data")

    // Rule Vector Reader
    val ruleVectorReaderData = Bits(OUTPUT, width=cp.RuleAmount)
    ruleVectorReaderData.setName("rule_vector_reader_data")
    val ruleVectorReaderCount = Bits(OUTPUT, width=log2Up(cp.RuleVectorAmount))
    ruleVectorReaderCount.setName("rule_vector_reader_count")
    val ruleVectorReaderRead = Bool(INPUT)
    ruleVectorReaderRead.setName("rule_vector_reader_read")
    val ruleVectorBufferReset = Bool(INPUT)
    ruleVectorBufferReset.setName("rule_vector_buffer_reset")

    // Rule Numbers Reader
    val ruleNumbersReaderAddressZ = Bits(INPUT, width=log2Up(cp.MatrixDepth))
    ruleNumbersReaderAddressZ.setName("rule_numbers_reader_address_z")
    val ruleNumbersReaderAddressY = Bits(INPUT, width=log2Up(cp.MatrixHeight))
    ruleNumbersReaderAddressY.setName("rule_numbers_reader_address_y")
    val ruleNumbersReaderData = Bits(OUTPUT, width=cp.MatrixWidth*log2Up(cp.RuleAmount))
    ruleNumbersReaderData.setName("rule_numbers_reader_data")

    val decodeOperation = UInt(INPUT, width=log2Up(Types.developmentOperation.size))
    decodeOperation.setName("decode_operation")
    val decodeRulesActive = Bits(INPUT, log2Up(cp.RuleAmount))
    decodeRulesActive.setName("decode_rules_active")

    val run = Bool(INPUT)
    run.setName("run")
    val done = Bool(OUTPUT)
    done.setName("done")
  }

  setVerilogParameters(new VerilogParameters {
    val matrix_width = cp.MatrixWidth
    val matrix_height = cp.MatrixHeight
    val matrix_depth = cp.MatrixDepth
    val matrix_wrap = cp.MatrixWrap
    val cell_type_bits = cp.CellTypeBits
    val cell_state_bits = cp.CellStateBits
    val rule_vector_amount = cp.RuleVectorAmount
    val rule_amount = cp.RuleAmount
    val rules_tested_in_parallel = cp.RulesTestedInParallel
  })
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
