package plan9

import Chisel._

class FetchIO(carpParams: CarpParameters) extends Bundle {
  val bufferData = Bits(INPUT, width=32)
  bufferData.setName("buffer_data")
  val bufferCount = Bits(INPUT, width=carpParams.RxBufferAddressBits)
  bufferCount.setName("buffer_count")
  val bufferRead = Bool(OUTPUT)
  bufferRead.setName("buffer_read")

  val instruction = Bits(OUTPUT, width=carpParams.InstructionBits)
  instruction.setName("instruction")

  val run = Bool(INPUT)
  run.setName("run")
  val done = Bool(OUTPUT)
  done.setName("done")
}

class Fetch(carpParams: CarpParameters) extends BlackBox {
  val io = new FetchIO(carpParams)

  setVerilogParameters(new VerilogParameters {
    val buffer_address_bits = carpParams.RxBufferAddressBits
    val jump_counters = carpParams.JumpCounters
    val jump_counter_bits = carpParams.JumpCounterBits
    val program_counter_bits = carpParams.ProgramCounterBits
    val instruction_bits = carpParams.InstructionBits
  })
  addClock(Driver.implicitClock)
  renameClock(Driver.implicitClock, "clock")
}
