package plan9

import Chisel._

class FitnessLiveCount(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle{
    val liveCountBufferRead = Bool(OUTPUT)
    liveCountBufferRead.setName("live_count_buffer_read")
    val liveCountBufferData = Bits(INPUT, width=cp.LiveCountBufferBits)
    liveCountBufferData.setName("live_count_buffer_data")
    val liveCountBufferCount = Bits(INPUT, width=log2Up(cp.LiveCountBufferSize))
    liveCountBufferCount.setName("live_count_buffer_count")

    val fitnessBufferWrite = Bool(OUTPUT)
    fitnessBufferWrite.setName("fitness_buffer_write")
    val fitnessBufferData = Bits(OUTPUT, width=32)
    fitnessBufferData.setName("fitness_buffer_data")
    val fitnessBufferCount = Bits(INPUT, width=log2Up(cp.FitnessBufferSize))
    fitnessBufferCount.setName("fitness_buffer_count")

    val identifier = Bits(OUTPUT, width=8)
    identifier.setName("identifier")
    val wordsPerRun = Bits(OUTPUT, width=8)
    wordsPerRun.setName("words_per_run")
    val parameters = Bits(OUTPUT, width=16)
    parameters.setName("parameters")
  }
  setVerilogParameters(new VerilogParameters{
    val live_count_buffer_size = cp.LiveCountBufferSize
    val live_count_buffer_bits = cp.LiveCountBufferBits
    val fitness_buffer_size = cp.FitnessBufferSize
  })
  setName("fitness_live_count")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
