package plan9

import Chisel._

class FitnessSender(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle {
    val sendBufferWrite = Bool(OUTPUT)
    sendBufferWrite.setName("send_buffer_write")
    val sendBufferData = Bits(OUTPUT, width=32)
    sendBufferData.setName("send_buffer_data")
    val sendBufferCount = Bits(INPUT, width=log2Up(cp.SendBufferSize))
    sendBufferCount.setName("send_buffer_count")

    val fitnessBufferRead = Bool(OUTPUT)
    fitnessBufferRead.setName("fitness_buffer_read")
    val fitnessBufferData = Bits(INPUT, width=32)
    fitnessBufferData.setName("fitness_buffer_data")
    val fitnessBufferCount = Bits(INPUT, width=log2Up(cp.FitnessBufferSize))
    fitnessBufferCount.setName("fitness_buffer_count")

    val fitnessWordsPerRun = Bits(INPUT, width=8)
    fitnessWordsPerRun.setName("fitness_words_per_run")

    val decodeOperation = UInt(INPUT, width=log2Up(Types.fitnessSenderOperation.size))
    decodeOperation.setName("decode_operation")

    val run = Bool(INPUT)
    run.setName("run")
    val done = Bool(OUTPUT)
    done.setName("done")
  }
  setVerilogParameters(new VerilogParameters {
    val send_buffer_size = cp.SendBufferSize
    val fitness_buffer_size = cp.FitnessBufferSize
  })
  setName("fitness_sender")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
