package plan9

import Chisel._

class LUTWriter(carpParams: CarpParameters) extends Module {
  val cp = carpParams
  val dataWidth = math.pow(2, (cp.NeighborhoodSize).toDouble).ceil.toInt

  val io = new Bundle {
    val lutStorageWrite = Bool(OUTPUT)
    val lutStorageAddress = Bits(OUTPUT, width=cp.CellTypeBits)
    val lutStorageData = Bits(OUTPUT, width=dataWidth)

    val decodeOperation = UInt(INPUT, width=log2Up(Types.lutWriterOperation.size))
    val decodeAddress = Bits(INPUT, width=cp.CellTypeBits)
    val decodeData = Bits(INPUT, width=dataWidth)

    val run = Bool(INPUT)
  }

  val addressReg = Reg(next = io.decodeAddress)
  val dataReg = Reg(next = io.decodeData)
  val writeReg = Reg(next = (io.run === Bool(true)
    && io.decodeOperation === Types.lutWriterOperation('store)
  ))

  io.lutStorageWrite := writeReg
  io.lutStorageAddress := addressReg
  io.lutStorageData := dataReg
}
