package plan9

import Chisel._
import util.Random

class Neuron(IncomingEdges: Int, WeightWidth: Int) extends Module {
  val io = new Bundle {
    val input = Bits(INPUT, width=IncomingEdges)

    val weightData = UInt(INPUT, width=WeightWidth)
    val weightAddr = UInt(INPUT, width=log2Up(IncomingEdges))
    val weightWrite = Bool(INPUT)

    val run = Bool(INPUT)
    val activationValue = Bool(OUTPUT)
  }

  val actValReg = Reg(init=Bool(false))
  io.activationValue := actValReg

  val weights = Vec.tabulate(IncomingEdges){i => Reg(init=UInt(0, width=WeightWidth))}
  val counts = Vec.tabulate(IncomingEdges){i => Reg(init=UInt(0, width=WeightWidth))}
  val newCounts = Vec.fill(IncomingEdges){UInt()}
  for (i <- 0 until IncomingEdges) {
    newCounts(i) := Mux(counts(i) === UInt(0), UInt(0), counts(i) - io.input(i))
  }

  when (io.weightWrite) {
    weights(io.weightAddr) := io.weightData.toUInt()
    counts(io.weightAddr) := io.weightData.toUInt()
  }.elsewhen (io.run) {
    when (newCounts.toBits() === Bits("b" + "0" * IncomingEdges)) {
      actValReg := Bool(true)
      for (i <- 0 until IncomingEdges) {
        counts(i) := weights(i)
      }
    } .otherwise {
      actValReg := Bool(false)
      for (i <- 0 until IncomingEdges) {
        counts(i) := newCounts(i)
      }
    }
  }
}
