package plan9

import Chisel._

class Readout(carpParams: CarpParameters) extends Module {
  val cp = carpParams

  require(log2Up(cp.SNNWeightCount()) <= cp.SNNWeightWidth)

  val io = new Bundle {
    val decodeOperation = UInt(INPUT, width=log2Up(Types.readoutOperation.size))
    val input = Bits(INPUT, width=cp.CAOutputCells.size)

    val output = Bits(OUTPUT, width=cp.SNNTopology(cp.SNNTopology.size-1))
    val outWrite = Bool(OUTPUT)

    val weightAddr = Bits(INPUT, width=log2Up(cp.SNNWeightCount()))
    val weightData = Bits(INPUT, width=cp.SNNWeightWidth)

    val run = Bool(INPUT)
  }

  val layers = Vec.tabulate(cp.SNNTopology.size){i =>
    Module(new SNNLayer(
      cp.SNNTopology(i),
      if (i == 0) cp.CAOutputCells.size else cp.SNNTopology(i-1),
      8
    )).io
  }

  var lo = 0
  var hi = cp.CAOutputCells.size * cp.SNNTopology(0)
  layers(0).input := io.input
  layers(0).run := io.run
  layers(0).weightData := io.weightData.toUInt()
  layers(0).weightAddr := io.weightAddr.toUInt() - UInt(lo)
  layers(0).weightWrite :=
    (io.weightAddr.toUInt() < UInt(hi)) &
    (io.decodeOperation === Types.readoutOperation('writeWeight))
  io.output := layers(layers.size-1).activationValues

  for (i <- 1 until layers.size) {
    lo = hi
    hi = hi + (cp.SNNTopology(i-1) * cp.SNNTopology(i))
    layers(i).input := layers(i-1).activationValues
    layers(i).weightData := io.weightData.toUInt()
    layers(i).weightAddr := io.weightAddr.toUInt() - UInt(lo)
    layers(i).weightWrite :=
      (io.weightAddr.toUInt() >= UInt(lo)) &
      (io.weightAddr.toUInt() < UInt(hi)) &
      (io.decodeOperation === Types.readoutOperation('writeWeight))
    layers(i).run := io.run
  }

  io.outWrite := Reg(next=io.run)
}
