package plan9

import Chisel._

class ReadoutSender(carpParams: CarpParameters) extends Module {
  val cp = carpParams
  val io = new Bundle {
    val sendBufferWrite = Bool(OUTPUT)
    val sendBufferData = Bits(OUTPUT, width=32)
    val sendBufferCount = Bits(INPUT, width=log2Up(cp.SendBufferSize))

    val readoutBufferRead = Bool(OUTPUT)
    val readoutBufferData = Bits(INPUT, width=1)
    val readoutBufferCount = Bits(INPUT, width=log2Up(cp.ReadoutBufferSize))

    val decodeOperation = UInt(INPUT, width=log2Up(Types.readoutSenderOperation.size))

    val run = Bool(INPUT)
    val done = Bool(OUTPUT)
  }

  val sIdle :: sWaitForReadoutData :: sSend :: Nil = Enum(UInt(), 3)
  val state = Reg(init=sIdle)

  val wordsPerSendOp = UInt(32)
  val readoutBufferHasData = io.readoutBufferCount.toUInt() >= wordsPerSendOp
  val sendBufferHasSpace = io.sendBufferCount.toUInt() < UInt(cp.SendBufferSize) - UInt(1) - wordsPerSendOp

  val iDone = Reg(init=Bool(true))
  val wordsToSend = Reg(init=wordsPerSendOp)

  // FSM and some internal/external controls
  switch (state) {
    is (sIdle) {
      when (io.run && io.decodeOperation === Types.readoutSenderOperation('send)) {
        state := sWaitForReadoutData
        iDone := Bool(false)
      }
    }
    is (sWaitForReadoutData) {
      when (readoutBufferHasData && sendBufferHasSpace) {
        state := sSend
      }
      wordsToSend := wordsPerSendOp
    }
    is (sSend) {
      when (wordsToSend === UInt(1)) {
        state := sIdle
        iDone := Bool(true)
      }
      wordsToSend := wordsToSend - UInt(1)
    }
  }

  // Transfer control
  when (state === sSend) {
    io.readoutBufferRead := Bool(true)
    io.sendBufferWrite := Bool(true)
  }.otherwise {
    io.readoutBufferRead := Bool(false)
    io.sendBufferWrite := Bool(false)
  }

  io.sendBufferData := Fill(31, UInt(0)) ## io.readoutBufferData
  io.done := iDone
}
