package plan9

import Chisel._

class RuleNumbersReader(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle {
    val bufferAddressZ = Bits(OUTPUT, width=log2Up(cp.MatrixDepth))
    bufferAddressZ.setName("buffer_address_z")
    val bufferAddressY = Bits(OUTPUT, width=log2Up(cp.MatrixHeight))
    bufferAddressY.setName("buffer_address_y")
    val bufferData = Bits(INPUT, width=cp.MatrixWidth*log2Up(cp.RuleAmount))
    bufferData.setName("buffer_data")

    val sendBufferData = Bits(OUTPUT, width=32)
    sendBufferData.setName("send_buffer_data")
    val sendBufferCount = Bits(INPUT, width=cp.TxBufferAddressBits)
    sendBufferCount.setName("send_buffer_count")
    val sendBufferWrite = Bool(OUTPUT)
    sendBufferWrite.setName("send_buffer_write")

    val decodeOperation = UInt(INPUT, width=log2Up(Types.ruleNumbersReaderOperation.size))
    decodeOperation.setName("decode_operation")

    val run = Bool(INPUT)
    run.setName("run")
    val done = Bool(OUTPUT)
    done.setName("done")
  }
  setVerilogParameters(new VerilogParameters {
    val matrix_width = cp.MatrixWidth
    val matrix_height = cp.MatrixHeight
    val matrix_depth = cp.MatrixDepth
    val rule_amount = cp.RuleAmount
    val send_buffer_address_bits = cp.TxBufferAddressBits
  })
  setName("rule_numbers_reader")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
