package plan9

import Chisel._

class RuleVectorReader(carpParams: CarpParameters) extends BlackBox {
  val cp = carpParams

  val io = new Bundle {
    val vectorBufferData = Bits(INPUT, width=cp.RuleAmount)
    vectorBufferData.setName("vector_buffer_data")
    val vectorBufferCount = Bits(INPUT, width=log2Up(cp.RuleVectorAmount))
    vectorBufferCount.setName("vector_buffer_count")
    val vectorBufferRead = Bool(OUTPUT)
    vectorBufferRead.setName("vector_buffer_read")

    val sendBufferData = Bits(OUTPUT, width=32)
    sendBufferData.setName("send_buffer_data")
    val sendBufferCount = Bits(INPUT, width=cp.TxBufferAddressBits)
    sendBufferCount.setName("send_buffer_count")
    val sendBufferWrite = Bool(OUTPUT)
    sendBufferWrite.setName("send_buffer_write")

    val decodeOperation = UInt(INPUT, width=log2Up(Types.ruleVectorReaderOperation.size))
    decodeOperation.setName("decode_operation")
    val decodeCount = Bits(INPUT, width=log2Up(cp.RuleVectorAmount))
    decodeCount.setName("decode_count")

    val run = Bool(INPUT)
    run.setName("run")
    val done = Bool(OUTPUT)
    done.setName("done")
  }

  setVerilogParameters(new VerilogParameters {
    val rule_amount = cp.RuleAmount
    val rule_vector_amount = cp.RuleVectorAmount
    val send_buffer_address_bits = cp.TxBufferAddressBits
  })
  setName("rule_vector_reader")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
