package plan9

import Chisel._

class RuleWriter(carpParams: CarpParameters) extends Module {
  val cp = carpParams
  val dataWidth = (cp.CellTypeBits + 1 + cp.CellStateBits + 1) * (cp.NeighborhoodSize + 1)
  val io = new Bundle {
    val ruleStorageWrite = Bool(OUTPUT)
    val ruleStorageAddress = Bits(OUTPUT, width=log2Up(cp.RuleAmount))
    val ruleStorageData = Bits(OUTPUT, width=dataWidth)

    val decodeOperation = UInt(INPUT, width=log2Up(Types.ruleWriterOperation.size))
    val decodeAddress = Bits(INPUT, width=log2Up(cp.RuleAmount))
    val decodeData = Bits(INPUT, width=dataWidth)

    val run = Bool(INPUT)
  }

  val addressReg = Reg(next = io.decodeAddress)
  val dataReg = Reg(next = io.decodeData)
  val writeReg = Reg(next = (io.run === Bool(true)
    && io.decodeOperation === Types.ruleWriterOperation('store)
  ))

  io.ruleStorageAddress := addressReg
  io.ruleStorageData := dataReg
  io.ruleStorageWrite := writeReg
}
