package plan9

import Chisel._
import util.Random

class SNNLayer(
  NeuronCount: Int,
  PreviousNeuronCount: Int,
  WeightWidth: Int
) extends Module {
  val io = new Bundle {
    val input = Bits(INPUT, width=PreviousNeuronCount)

    val weightData = UInt(INPUT, width=WeightWidth)
    val weightAddr = UInt(INPUT, width=log2Up(NeuronCount*PreviousNeuronCount))
    val weightWrite = Bool(INPUT)

    val run = Bool(INPUT)
    val activationValues = Bits(OUTPUT, width=NeuronCount)
  }

  io.activationValues := Bits(0)

  val neurons = Vec.tabulate(NeuronCount){i => Module(new Neuron(
    PreviousNeuronCount,
    WeightWidth
  )).io}

  for (i <- 0 until NeuronCount) {
    neurons(i).run := io.run
    neurons(i).input := io.input
    neurons(i).weightData := io.weightData
    neurons(i).weightAddr := io.weightAddr % UInt(PreviousNeuronCount)
    neurons(i).weightWrite := ((io.weightAddr / UInt(PreviousNeuronCount)) === UInt(i)) & io.weightWrite

    io.activationValues(i) := neurons(i).activationValue
  }
}
