package plan9

import Chisel._

class SendBufferMux(carpParams: CarpParameters) extends BlackBox {
  val SendBufferAddressBits = carpParams.TxBufferAddressBits

  val io = new Bundle {
    // Cell Writer Reader
    val cellWriterReaderData = Bits(INPUT, width=32)
    cellWriterReaderData.setName("cell_writer_reader_data")
    val cellWriterReaderWrite = Bool(INPUT)
    cellWriterReaderWrite.setName("cell_writer_reader_write")
    val cellWriterReaderCount = Bits(OUTPUT, width=SendBufferAddressBits)
    cellWriterReaderCount.setName("cell_writer_reader_count")

    // Information Sender
    val informationSenderData = Bits(INPUT, width=32)
    informationSenderData.setName("information_sender_data")
    val informationSenderWrite = Bool(INPUT)
    informationSenderWrite.setName("information_sender_write")
    val informationSenderCount = Bits(OUTPUT, width=SendBufferAddressBits)
    informationSenderCount.setName("information_sender_count")

    // Rule Vector Reader
    val ruleVectorReaderData = Bits(INPUT, width=32)
    ruleVectorReaderData.setName("rule_vector_reader_data")
    val ruleVectorReaderWrite = Bool(INPUT)
    ruleVectorReaderWrite.setName("rule_vector_reader_write")
    val ruleVectorReaderCount = Bits(OUTPUT, width=SendBufferAddressBits)
    ruleVectorReaderCount.setName("rule_vector_reader_count")

    // Rule Numbers Reader
    val ruleNumbersReaderData = Bits(INPUT, width=32)
    ruleNumbersReaderData.setName("rule_numbers_reader_data")
    val ruleNumbersReaderWrite = Bool(INPUT)
    ruleNumbersReaderWrite.setName("rule_numbers_reader_write")
    val ruleNumbersReaderCount = Bits(OUTPUT, width=SendBufferAddressBits)
    ruleNumbersReaderCount.setName("rule_numbers_reader_count")

    // Fitness Sender
    val fitnessSenderData = Bits(INPUT, width=32)
    fitnessSenderData.setName("fitness_sender_data")
    val fitnessSenderWrite = Bool(INPUT)
    fitnessSenderWrite.setName("fitness_sender_write")
    val fitnessSenderCount = Bits(OUTPUT, width=SendBufferAddressBits)
    fitnessSenderCount.setName("fitness_sender_count")

    // Readout Sender
    val readoutSenderData = Bits(INPUT, width=32)
    readoutSenderData.setName("readout_sender_data")
    val readoutSenderWrite = Bool(INPUT)
    readoutSenderWrite.setName("readout_sender_write")
    val readoutSenderCount = Bits(OUTPUT, width=SendBufferAddressBits)
    readoutSenderCount.setName("readout_sender_count")

    // Buffer
    val sendBufferData = Bits(OUTPUT, width=32)
    sendBufferData.setName("send_buffer_data")
    val sendBufferWrite = Bool(OUTPUT)
    sendBufferWrite.setName("send_buffer_write")
    val sendBufferCount = Bits(INPUT, width=SendBufferAddressBits)
    sendBufferCount.setName("send_buffer_count")

    val sourceSelect = UInt(INPUT, width=log2Up(Types.sendBufferMuxSelect.size))
    sourceSelect.setName("source_select")

    val run = Bool(INPUT)
    run.setName("run")
  }

  setVerilogParameters(new VerilogParameters {
    val send_buffer_address_bits = SendBufferAddressBits
  })
  setName("send_buffer_mux")
  addClock(Driver.implicitClock)
  renameClock("clk", "clock")
}
