# Author: Asbjørn Djupdal
# Improved by Benjamin Bjørnseth and Anders T. Akre

vlib work

#####  IMPORTANT #####
# To run modelsim, you need to obtain the libraries vmap'ed below. The libraries
# are assumed to be located in your home folder, under lib/compile_simlib
######################

set home $::env(HOME)

set includes +incdir+../src/verilog/

#Map the required libraries here#
vmap secureip /opt/compile_simlib/secureip
vmap unimacro /opt/compile_simlib/unimacro
#vmap unifast /opt/compile_simlib/unifast
vmap unisims_ver /opt/compile_simlib/unisims_ver
vmap unisim /opt/compile_simlib/unisim
vmap unimacro_ver /opt/compile_simlib/unimacro_ver
#vmap unifast_ver /opt/compile_simlib/unifast_ver
vmap simprims_ver /opt/compile_simlib/simprims_ver

# compile
vcom ../vhdl/packages/functions.vhd
vcom ../vhdl/packages/instructions.vhd
vcom ../vhdl/packages/types.vhd
vcom ../vhdl/modules/information_sender.vhd
vcom ../vhdl/modules/fitness_sender.vhd
vcom ../vhdl/modules/utility/selector.vhd
vcom ../vhdl/modules/utility/bit_counter_32.vhd
vcom ../vhdl/modules/utility/bram_tdp.vhd
vcom ../vhdl/modules/utility/shift_register.vhd
vcom ../vhdl/modules/utility/fifo_synchronizer.vhd
vcom ../vhdl/modules/utility/shifter.vhd
vcom ../vhdl/modules/utility/shifter_dynamic.vhd
vcom ../vhdl/modules/utility/combiner.vhd
vcom ../vhdl/modules/utility/fifo.vhd
vcom ../vhdl/modules/utility/bit_counter_N.vhd
vcom ../vhdl/modules/utility/bram_1toN.vhd
vcom ../vhdl/modules/send_buffer_mux.vhd
vcom ../vhdl/modules/resetter.vhd
vcom ../vhdl/modules/cell_storage_mux.vhd
vcom ../vhdl/modules/cell_writer_reader.vhd
vcom ../vhdl/modules/rule_vector_reader.vhd
vcom ../vhdl/modules/cell_storage.vhd
vcom ../vhdl/modules/rule_numbers_reader.vhd
vcom ../vhdl/modules/fetch/fetch_handler.vhd
vcom ../vhdl/modules/fetch/fetch_communication.vhd
vcom ../vhdl/modules/fetch/fetch.vhd
vcom ../vhdl/modules/cellular_automata/live_counter.vhd
vcom ../vhdl/modules/cellular_automata/lut_configurable.vhd
vcom ../vhdl/modules/cellular_automata/sblock.vhd
vcom ../vhdl/modules/cellular_automata/sblock_matrix.vhd
vcom ../vhdl/modules/cellular_automata/cellular_automata.vhd
vcom ../vhdl/modules/development/hits_to_numbers.vhd
vcom ../vhdl/modules/development/cell_fetcher.vhd
vcom ../vhdl/modules/development/rule_tester.vhd
vcom ../vhdl/modules/development/rule_tester_multi.vhd
vcom ../vhdl/modules/development/rule_testers_multi.vhd
vcom ../vhdl/modules/development/hits_to_vector.vhd
vcom ../vhdl/modules/development/rule_fetcher.vhd
vcom ../vhdl/modules/development/development.vhd
vcom ../vhdl/modules/fitness/fitness_live_count.vhd
vcom ../vhdl/modules/decode.vhd
vlog $includes ../verilog/ConveyApbBridge.v
vlog $includes ../verilog/DualPortBRAM.v
vlog $includes ../verilog/Q_srl.v
vlog $includes testbench_aeg.v

vlog $includes glbl.v

#Load the design. Use required libraries.#
vsim +notimingchecks          \
         -L unisims_ver       \
         -L unisim            \
         -L secureip          \
         -L simprims_ver      \
         -L unimacro_ver      \
         -voptargs=+acc       \
         work.plan9_tb work.glbl

#Change radix to Hexadecimal#
radix hex

#Supress Numeric Std package and Arith package warnings.#
#For VHDL designs we get some warnings due to unknown values on some signals at startup#
# ** Warning: NUMERIC_STD.TO_INTEGER: metavalue detected, returning 0#
#We may also get some Arithmetic packeage warnings because of unknown values on#
#some of the signals that are used in an Arithmetic operation.#
#In order to suppress these warnings, we use following two commands#

set NumericStdNoWarnings 1
set StdArithNoWarnings 1
