`timescale 1ps / 1ps

`define REG_RD_DATA 0
`define REG_RD_TX_BUFFER_COUNT 1
`define REG_RD_TX_BUFFER_SPACE 2
`define REG_RD_RX_BUFFER_COUNT 3
`define REG_RD_RX_BUFFER_SPACE 4
`define REG_RD_MAGIC 5
`define REG_RD_WRITE_TO_MEM 6
`define REG_RD_MEM 7

`define REG_WR_DATA 0
`define REG_WR_READ_FROM_MEM 1
`define REG_WR_START_CMD 2
`define REG_WR_END_CMD 3
`define REG_WR_START_DATA 4
`define REG_WR_END_DATA 5
`define REG_WR_MEM 6

`define MC_READ 1
`define MC_WRITE 2

module plan9_tb;
   
   reg          clk; 
   reg          reset;
   reg          io_clkhx;
   reg          io_clk2x;
   reg          io_disp_inst_vld;
   reg [4:0]    io_disp_inst;
   reg [17:0]   io_disp_aeg_idx;
   reg          io_disp_aeg_rd;
   reg          io_disp_aeg_wr;
   reg [63:0]   io_disp_aeg_wr_data;
   wire [17:0]  io_disp_aeg_cnt;
   wire [15:0]  io_disp_exception;
   wire         io_disp_idle;
   wire         io_disp_rtn_data_vld;
   wire [63:0]  io_disp_rtn_data;
   wire         io_disp_stall;
   wire         io_mc_rq_vld;
   wire [47:0]  io_mc_rq_rtnctl;
   wire [63:0]  io_mc_rq_data;
   wire [47:0]  io_mc_rq_vadr;
   wire [1:0]   io_mc_rq_size;
   wire [2:0]   io_mc_rq_cmd;
   wire [3:0]   io_mc_rq_scmd;
   reg          io_mc_rq_stall;
   reg          io_mc_rs_vld;
   reg [2:0]    io_mc_rs_cmd;
   reg [3:0]    io_mc_rs_scmd;
   reg [63:0]   io_mc_rs_data;
   reg [47:0]   io_mc_rs_rtnctl;
   wire         io_mc_rs_stall;
   wire         io_mc_rq_flush;
   reg          io_mc_rs_flush_cmplt;
   reg          io_csr_wr_vld;
   reg          io_csr_rd_vld;
   reg [15:0]   io_csr_address;
   reg [63:0]   io_csr_wr_data;
   wire         io_csr_rd_ack;
   wire [63:0]  io_csr_rd_data;
   wire [3:0]   io_i_aeid;

   ConveyApbBridge dut
     (
      .clk (clk),
      .reset (reset),
      .io_clkhx (io_clkhx),
      .io_clk2x (io_clk2x),
      .io_disp_inst_vld (io_disp_inst_vld),
      .io_disp_inst (io_disp_inst),
      .io_disp_aeg_idx (io_disp_aeg_idx),
      .io_disp_aeg_rd (io_disp_aeg_rd),
      .io_disp_aeg_wr (io_disp_aeg_wr),
      .io_disp_aeg_wr_data (io_disp_aeg_wr_data),
      .io_disp_aeg_cnt (io_disp_aeg_cnt),
      .io_disp_exception (io_disp_exception),
      .io_disp_idle (io_disp_idle),
      .io_disp_rtn_data_vld (io_disp_rtn_data_vld),
      .io_disp_rtn_data (io_disp_rtn_data),
      .io_disp_stall (io_disp_stall),
      .io_mc_rq_vld (io_mc_rq_vld),
      .io_mc_rq_rtnctl (io_mc_rq_rtnctl),
      .io_mc_rq_data (io_mc_rq_data),
      .io_mc_rq_vadr (io_mc_rq_vadr),
      .io_mc_rq_size (io_mc_rq_size),
      .io_mc_rq_cmd (io_mc_rq_cmd),
      .io_mc_rq_scmd (io_mc_rq_scmd),
      .io_mc_rq_stall (io_mc_rq_stall),
      .io_mc_rs_vld (io_mc_rs_vld),
      .io_mc_rs_cmd (io_mc_rs_cmd),
      .io_mc_rs_scmd (io_mc_rs_scmd),
      .io_mc_rs_data (io_mc_rs_data),
      .io_mc_rs_rtnctl (io_mc_rs_rtnctl),
      .io_mc_rs_stall (io_mc_rs_stall),
      .io_mc_rq_flush (io_mc_rq_flush),
      .io_mc_rs_flush_cmplt (io_mc_rs_flush_cmplt),
      .io_csr_wr_vld (io_csr_wr_vld),
      .io_csr_rd_vld (io_csr_rd_vld),
      .io_csr_address (io_csr_address),
      .io_csr_wr_data (io_csr_wr_data),
      .io_csr_rd_ack (io_csr_rd_ack),
      .io_csr_rd_data (io_csr_rd_data),
      .io_i_aeid (io_i_aeid)
      );

   reg [31:0]   data;
   reg [63:0]   mem_data;
   reg [47:0]   mem_addr;

   reg [63:0]   mem[0:1023];

   integer      i, displayWrites;

   initial begin 
      clk = 0; 
      io_clkhx = 0; 
      io_clk2x = 0; 
   end 
   
   always 
     #10000 clk = !clk; 

   always 
     #20000 io_clkhx = !io_clkhx;

   always 
     #5000 io_clk2x = !io_clk2x;

   task apb_write;
      input [31:0] addr;
      input [63:0] data;

      begin
        @(posedge clk);
        io_disp_aeg_idx = addr;
        io_disp_aeg_rd = 0;
        io_disp_aeg_wr = 1;
        io_disp_aeg_wr_data = data;

        @(posedge clk);
        
        while(io_disp_stall) begin
           @(posedge clk);
        end

        io_disp_aeg_wr = 0;
      end
   endtask

   task apb_read;
      input [31:0] addr;
      output [63:0] data;

      begin
        @(posedge clk);
        io_disp_aeg_idx = addr;
        io_disp_aeg_rd = 1;
        io_disp_aeg_wr = 0;

        @(posedge clk);
        
        while(io_disp_stall) begin
           @(posedge clk);
        end

        io_disp_aeg_rd = 0;

        while(!io_disp_rtn_data_vld) begin
           @(posedge clk);
        end

        data = io_disp_rtn_data;
      end
   endtask

   initial begin
      mem[0] = 64'h0000000000000069;
      mem[1] = 64'h0000000000000001;
      mem[2] = 64'h0000000000000C17;
      mem[3] = 64'h0000000000000000;
      mem[4] = 64'h0000000000000069;
      mem[5] = 64'h0000000000000002;
      mem[6] = 64'h000000000000141D;
      mem[7] = 64'h0000000000000000;
      mem[8] = 64'h000000000002000A;
      mem[9] = 64'h000000000000002C;
      mem[10] = 64'h0000000000000001;
      mem[11] = 64'h000000000000002E;
      mem[12] = 64'h0000000000000000;
      mem[13] = 64'h000000000000012C;
      mem[14] = 64'h0000000000000000;
      mem[15] = 64'h000000000000012E;
      mem[16] = 64'h0000000000000001;
      mem[17] = 64'h000000000000022C;
      mem[18] = 64'h0000000000000001;
      mem[19] = 64'h000000000000022E;
      mem[20] = 64'h0000000000000002;
      mem[21] = 64'h000000000000032C;
      mem[22] = 64'h0000000000000000;
      mem[23] = 64'h000000000000032E;
      mem[24] = 64'h0000000000000003;
      mem[25] = 64'h0000000000000010;
      mem[26] = 64'h0000000000000014;
      mem[27] = 64'h0000000000000004;
      mem[28] = 64'h0000000000000104;
      mem[29] = 64'h0000000000000204;
      mem[30] = 64'h0000000000000304;
      mem[31] = 64'h0000000000000006;
      mem[32] = 64'h0000000000000106;
      mem[33] = 64'h0000000000000206;
      mem[34] = 64'h0000000000000306;
      mem[35] = 64'h0000000000010002;
      mem[36] = 64'h0000000000000003;

      while(1) begin
         io_mc_rq_stall = 0;
         io_mc_rs_vld = 0;
         io_mc_rs_cmd = `MC_READ;
         io_mc_rs_scmd = 0;
         io_mc_rs_data = 0;
         io_mc_rs_rtnctl = 0;

         if(io_mc_rq_vld) begin
            if(io_mc_rq_cmd == `MC_WRITE) begin
               mem[io_mc_rq_vadr>>3] = io_mc_rq_data;
               if(displayWrites == 1) begin
                  $display("Write %x\n", io_mc_rq_data);
               end
            end else begin
               mem_data = mem[io_mc_rq_vadr>>3];
               mem_addr = io_mc_rq_vadr;

               while(io_mc_rs_stall) begin
                  @(posedge clk);
               end

               io_mc_rs_vld = 1;
               io_mc_rs_data = mem_data;
               io_mc_rs_rtnctl = mem_addr;
            end
         end

         @(posedge clk);
      end
   end

   initial begin
      while(1) begin
         io_mc_rs_flush_cmplt = 0;

         if(io_mc_rq_flush) begin
            @(posedge clk);
            io_mc_rs_flush_cmplt = 1;
         end

         @(posedge clk);
      end
   end

   initial
     begin
        $display("Starting testbench");

        reset = 1;

        io_disp_inst_vld = 0;
        io_disp_inst = 0;

        io_disp_aeg_idx = 0;
        io_disp_aeg_rd = 0;
        io_disp_aeg_wr = 0;
        io_disp_aeg_wr_data = 0;

        io_csr_wr_vld = 0;
        io_csr_rd_vld = 0;
        io_csr_address = 0;
        io_csr_wr_data = 0;

        displayWrites = 0;

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        reset = 0;

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        apb_read(`REG_RD_MAGIC, data);
        if(data != 32'h01234567) $fatal("magic");

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        apb_write(`REG_WR_START_CMD, 0<<3);
        apb_write(`REG_WR_END_CMD, 37<<3);
        apb_write(`REG_WR_READ_FROM_MEM, 0);

        displayWrites = 1;

        for(i = 0; i < 12; i = i + 1) begin
          //apb_write(`REG_WR_START_DATA, 10<<3);
          //apb_write(`REG_WR_END_DATA, 11<<3);
          //apb_read(`REG_RD_WRITE_TO_MEM, data);
          apb_read(`REG_RD_DATA, data);
          $display("Data: %x\n", data);
        end

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        $stop;

     end

endmodule
