`timescale 1ps / 1ps

`define REG_RD_DATA 0
`define REG_RD_TX_BUFFER_COUNT 4
`define REG_RD_TX_BUFFER_SPACE 8
`define REG_RD_RX_BUFFER_COUNT 12
`define REG_RD_RX_BUFFER_SPACE 16
`define REG_RD_MAGIC 20
`define REG_RD_WAIT_FOR_DATA 24
`define REG_RD_MEM 28

`define REG_WR_DATA 0
`define REG_WR_CONTROL 4
`define REG_WR_START_CMD 8
`define REG_WR_END_CMD 12
`define REG_WR_START_DATA 16
`define REG_WR_END_DATA 20
`define REG_WR_MEM 24

module plan9_tb;
   
   reg          clk; 
   reg          reset;

   reg [31:0]   io_apb_PADDR;
   reg          io_apb_PWRITE;
   reg          io_apb_PSEL;
   reg          io_apb_PENABLE;
   reg [31:0]   io_apb_PWDATA;
   wire [31:0]  io_apb_PRDATA;
   wire         io_apb_PREADY;
   wire         io_apb_PSLVERR;
   wire         io_apb_IRQ;

   reg          io_mem_req_ready;
   wire         io_mem_req_valid;
   wire [31:0]  io_mem_req_bits_addr;
   wire         io_mem_req_bits_write;
   wire [31:0]  io_mem_req_bits_data;
   wire         io_mem_resp_ready;
   reg          io_mem_resp_valid;
   reg [31:0]   io_mem_resp_bits_addr;
   reg          io_mem_resp_bits_write;
   reg [31:0]   io_mem_resp_bits_data;
   wire         io_mem_flush;
   reg          io_mem_flush_complete;

   reg [31:0]   data;
   reg [31:0]   mem_data;
   reg [31:0]   mem_addr;

   reg [31:0]   mem[0:1024];

   Toplevel dut
     (
      .clk (clk),
      .reset (reset),
      .io_apb_PADDR (io_apb_PADDR),
      .io_apb_PWRITE (io_apb_PWRITE),
      .io_apb_PSEL (io_apb_PSEL),
      .io_apb_PENABLE (io_apb_PENABLE),
      .io_apb_PWDATA (io_apb_PWDATA),
      .io_apb_PRDATA (io_apb_PRDATA),
      .io_apb_PREADY (io_apb_PREADY),
      .io_apb_PSLVERR (io_apb_PSLVERR),
      .io_apb_IRQ (io_apb_IRQ),
      .io_mem_req_ready (io_mem_req_ready),
      .io_mem_req_valid (io_mem_req_valid),
      .io_mem_req_bits_addr (io_mem_req_bits_addr),
      .io_mem_req_bits_write (io_mem_req_bits_write),
      .io_mem_req_bits_data (io_mem_req_bits_data),
      .io_mem_resp_ready (io_mem_resp_ready),
      .io_mem_resp_valid (io_mem_resp_valid),
      .io_mem_resp_bits_addr (io_mem_resp_bits_addr),
      .io_mem_resp_bits_write (io_mem_resp_bits_write),
      .io_mem_resp_bits_data (io_mem_resp_bits_data),
      .io_mem_flush (io_mem_flush),
      .io_mem_flush_complete (io_mem_flush_complete)
      );

   initial begin
      while(1) begin
         io_mem_req_ready = 1;
         io_mem_resp_valid = 0;
         io_mem_resp_bits_addr = 0;
         io_mem_resp_bits_write = 0;
         io_mem_resp_bits_data = 0;

         if(io_mem_req_valid) begin
            if(io_mem_req_bits_write) begin
               mem[io_mem_req_bits_addr] = io_mem_req_bits_data;
            end else begin
               mem_data = mem[io_mem_req_bits_addr];
               mem_addr = io_mem_req_bits_addr;
               @(posedge clk);
               io_mem_resp_valid = 1;
               io_mem_resp_bits_data = mem_data;
               io_mem_resp_bits_addr = mem_addr;
            end
         end
         @(posedge clk);
      end
         
   end

   initial begin 
      clk = 0; 
   end 
   
   always 
     #10000 clk = !clk; 

   task apb_write;
      input [31:0] addr;
      input [31:0] data;

      begin
        @(posedge clk);
        io_apb_PADDR = addr;
        io_apb_PWRITE = 1'b1;
        io_apb_PSEL = 1'b1;
        io_apb_PWDATA = data;

        @(posedge clk);
        io_apb_PENABLE = 1'b1;
        
        @(posedge clk);
        
        while(!io_apb_PREADY) begin
           @(posedge clk);
        end

        io_apb_PENABLE = 1'b0;
        io_apb_PSEL = 1'b0;
      end
   endtask

   task apb_read;
      input [31:0] addr;
      output [31:0] data;

      begin
        @(posedge clk);
        io_apb_PADDR = addr;
        io_apb_PWRITE = 1'b0;
        io_apb_PSEL = 1'b1;

        @(posedge clk);
        io_apb_PENABLE = 1'b1;
        
        @(posedge clk);
        
        while(!io_apb_PREADY) begin
           @(posedge clk);
        end

        data = io_apb_PRDATA;

        io_apb_PENABLE = 1'b0;
        io_apb_PSEL = 1'b0;
      end
   endtask

   initial
     begin
        $display("Starting testbench");

        reset = 1;

        io_apb_PADDR = 0;
        io_apb_PWRITE = 0;
        io_apb_PSEL = 0;
        io_apb_PENABLE = 0;
        io_apb_PWDATA = 0;

        io_mem_req_ready = 1;
        io_mem_resp_valid = 0;
        io_mem_resp_bits_addr = 0;
        io_mem_resp_bits_write = 0;
        io_mem_resp_bits_data = 0;
        io_mem_flush_complete = 1;

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        reset = 0;

        @(posedge clk);

        apb_read(`REG_RD_MAGIC, data);
        $display("data: %x", data);
        if(data != 32'h01234567) $fatal("magic");

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        apb_write(`REG_WR_START, 32'h00000010);
        apb_write(`REG_WR_MEM, 32'haabbccdd);

        apb_write(`REG_WR_START, 32'h00000014);
        apb_write(`REG_WR_MEM, 32'h11223344);

        apb_write(`REG_WR_START, 32'h00000010);
        apb_read(`REG_RD_MEM, data);
        if(data != 32'haabbccdd) $fatal("mem");

        apb_write(`REG_WR_START, 32'h00000014);
        apb_read(`REG_RD_MEM, data);
        if(data != 32'h11223344) $fatal("mem");

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        apb_write(`REG_WR_CONTROL, 32'h00000001);

        apb_write(`REG_WR_START, 32'h00000010);
        apb_write(`REG_WR_END, 32'h00000020);

        apb_write(`REG_WR_DATA, 32'h00000089);
        apb_write(`REG_WR_DATA, 32'h00000001);
        apb_write(`REG_WR_DATA, 32'h00006017);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h00000089);
        apb_write(`REG_WR_DATA, 32'h00000002);
        apb_write(`REG_WR_DATA, 32'h0000A01D);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h0002000A);
        apb_write(`REG_WR_DATA, 32'h0000002C);
        apb_write(`REG_WR_DATA, 32'h00000001);
        apb_write(`REG_WR_DATA, 32'h0000002E);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h0000012C);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h0000012E);
        apb_write(`REG_WR_DATA, 32'h00000001);
        apb_write(`REG_WR_DATA, 32'h0000022C);
        apb_write(`REG_WR_DATA, 32'h00000001);
        apb_write(`REG_WR_DATA, 32'h0000022E);
        apb_write(`REG_WR_DATA, 32'h00000002);
        apb_write(`REG_WR_DATA, 32'h0000032C);
        apb_write(`REG_WR_DATA, 32'h00000000);
        apb_write(`REG_WR_DATA, 32'h0000032E);
        apb_write(`REG_WR_DATA, 32'h00000003);
        apb_write(`REG_WR_DATA, 32'h00000010);
        apb_write(`REG_WR_DATA, 32'h00000014);
        apb_write(`REG_WR_DATA, 32'h00000004);
        apb_write(`REG_WR_DATA, 32'h00000104);
        apb_write(`REG_WR_DATA, 32'h00000204);
        apb_write(`REG_WR_DATA, 32'h00000304);
        apb_write(`REG_WR_DATA, 32'h00000006);
        apb_write(`REG_WR_DATA, 32'h00000106);
        apb_write(`REG_WR_DATA, 32'h00000206);
        apb_write(`REG_WR_DATA, 32'h00000306);
        apb_write(`REG_WR_DATA, 32'h00010002);
        apb_write(`REG_WR_DATA, 32'h00000003);

        @(posedge clk);
        @(posedge clk);
        @(posedge clk);

        apb_read(`REG_RD_WAIT_FOR_DATA, data);

        $stop;
     end

endmodule
