`timescale 1 ns / 1 ps

`include "pdk_fpga_defines.vh"

(* keep_hierarchy = "true" *)
module cae_pers #(
   parameter    NUM_MC_PORTS = 8,
   parameter    RTNCTL_WIDTH = 32
) (
   //
   // Clocks and Resets
   //
   input wire                                  clk, // Personalitycore clock
   input wire                                  clkhx, // half-rate clock
   input wire                                  clk2x, // 2x rate clock
   input wire                                  i_reset, // global reset synchronized to clk

   //
   // Dispatch Interface
   //
   input wire                                  disp_inst_vld,
   input wire [4:0]                            disp_inst,
   input wire [17:0]                           disp_aeg_idx,
   input wire                                  disp_aeg_rd,
   input wire                                  disp_aeg_wr,
   input wire [63:0]                           disp_aeg_wr_data,

   output wire [17:0]                          disp_aeg_cnt,
   output wire [15:0]                          disp_exception,
   output wire                                 disp_idle,
   output wire                                 disp_rtn_data_vld,
   output wire [63:0]                          disp_rtn_data,
   output wire                                 disp_stall,

   //
   // MC Interface(s)
   //
   output wire [NUM_MC_PORTS*1-1 :0]           mc_rq_vld,
   output wire [NUM_MC_PORTS*RTNCTL_WIDTH-1:0] mc_rq_rtnctl,
   output wire [NUM_MC_PORTS*64-1:0]           mc_rq_data,
   output wire [NUM_MC_PORTS*48-1:0]           mc_rq_vadr,
   output wire [NUM_MC_PORTS*2-1 :0]           mc_rq_size,
   output wire [NUM_MC_PORTS*3-1 :0]           mc_rq_cmd,
   output wire [NUM_MC_PORTS*4-1 :0]           mc_rq_scmd,
   input wire [NUM_MC_PORTS*1-1 :0]            mc_rq_stall,
   
   input wire [NUM_MC_PORTS*1-1 :0]            mc_rs_vld,
   input wire [NUM_MC_PORTS*3-1 :0]            mc_rs_cmd,
   input wire [NUM_MC_PORTS*4-1 :0]            mc_rs_scmd,
   input wire [NUM_MC_PORTS*64-1:0]            mc_rs_data,
   input wire [NUM_MC_PORTS*RTNCTL_WIDTH-1:0]  mc_rs_rtnctl,
   output wire [NUM_MC_PORTS*1-1 :0]           mc_rs_stall,

   // Write flush 
   output wire [NUM_MC_PORTS*1-1 :0]           mc_rq_flush,
   input wire [NUM_MC_PORTS*1-1 :0]            mc_rs_flush_cmplt,

   //
   // Management/Debug Interface
   //
   input wire                                  csr_wr_vld,
   input wire                                  csr_rd_vld,
   input wire [15:0]                           csr_address,
   input wire [63:0]                           csr_wr_data,
   output wire                                 csr_rd_ack,
   output wire [63:0]                          csr_rd_data,

   //
   // Miscellaneous
   //
   input wire [3:0]                            i_aeid
);

`include "pdk_fpga_param.vh"

   //**************************************************************************
   //			   PERSONALITY SPECIFIC LOGIC
   //**************************************************************************

   wire r_reset;
   FDSE rst (.C(clk),.S(i_reset),.CE(r_reset),.D(!r_reset),.Q(r_reset));

   ConveyApbBridge bridge (.clk (clk),
                           .reset (r_reset),
                           .io_clkhx (clkhx),
                           .io_clk2x (clk2x),
                           .io_disp_inst_vld (disp_inst_vld),
                           .io_disp_inst (disp_inst),
                           .io_disp_aeg_idx (disp_aeg_idx),
                           .io_disp_aeg_rd (disp_aeg_rd),
                           .io_disp_aeg_wr (disp_aeg_wr),
                           .io_disp_aeg_wr_data (disp_aeg_wr_data),
                           .io_disp_aeg_cnt (disp_aeg_cnt),
                           .io_disp_exception (disp_exception),
                           .io_disp_idle (disp_idle),
                           .io_disp_rtn_data_vld (disp_rtn_data_vld),
                           .io_disp_rtn_data (disp_rtn_data),
                           .io_disp_stall (disp_stall),
                           .io_mc_rq_vld (mc_rq_vld),
                           .io_mc_rq_rtnctl (mc_rq_rtnctl),
                           .io_mc_rq_data (mc_rq_data),
                           .io_mc_rq_vadr (mc_rq_vadr),
                           .io_mc_rq_size (mc_rq_size),
                           .io_mc_rq_cmd (mc_rq_cmd),
                           .io_mc_rq_scmd (mc_rq_scmd),
                           .io_mc_rq_stall (mc_rq_stall),
                           .io_mc_rs_vld (mc_rs_vld),
                           .io_mc_rs_cmd (mc_rs_cmd),
                           .io_mc_rs_scmd (mc_rs_scmd),
                           .io_mc_rs_data (mc_rs_data),
                           .io_mc_rs_rtnctl (mc_rs_rtnctl),
                           .io_mc_rs_stall (mc_rs_stall),
                           .io_mc_rq_flush (mc_rq_flush),
                           .io_mc_rs_flush_cmplt (mc_rs_flush_cmplt),
                           .io_csr_wr_vld (csr_wr_vld),
                           .io_csr_rd_vld (csr_rd_vld),
                           .io_csr_address (csr_address),
                           .io_csr_wr_data (csr_wr_data),
                           .io_csr_rd_ack (csr_rd_ack),
                           .io_csr_rd_data (csr_rd_data),
                           .io_i_aeid (i_aeid)
                           );

endmodule // cae_pers
