/*****************************************************************************
 * CARP Types
 *
 * Types for working with the CARP API
 *
 * Øyvind Robertsen 2017
 *****************************************************************************/

#ifndef TYPES_H
#define TYPES_H

#include <stdint.h>
#include <stdbool.h>

typedef struct {
  bool    matrix_wrap;
  uint8_t matrix_width;
  uint8_t matrix_height;
  uint8_t matrix_depth;

  uint8_t state_bits;
  uint8_t type_bits;
  uint8_t counter_amount;
  uint8_t counter_bits;

  uint32_t rule_amount;

  uint8_t fitness_id;
  uint8_t fitness_words;
  uint16_t fitness_params;

  uint16_t ca_output_cell_count;
  uint16_t readout_layer_count;
  uint16_t* readout_topology;
} carp_info_t;

typedef struct {
  uint8_t* weights;
} neuron_t;

typedef struct {
  neuron_t* neurons;
  uint16_t neuron_count;
} snn_layer_t;

typedef struct {
  snn_layer_t* layers;
  uint16_t layer_count;
  uint16_t input_count;
} snn_t;

#endif
