#include <stdlib.h>
#include "print.h"
#include "postscript.h"

carp_info_t *info;

uint32_t colorfunction (uint32_t value) {
  if (value <= 0) {
    return 0xFFFFFF;
  }
  if (value <= 8) {
    return 0x33FF33;
  }
  if (value <= 9) {
    return 0xFF6666;
  }
  if (value <= 0xD) {
    return 0xFFFF33;
  }
  return 0xFFFFFF;
}

void demo() {
  printf("Demo: Readout Feedback\n");

  write_lut(LUT_SELF, 0);
  write_lut(LUT_XOR,  1);
  write_lut(LUT_OR,   2);

  for (int y = 0; y < 8; y++) {
    for (int x = 0; x < 8; x++) {
      write_type(0, y, x, 1);
    }
  }

  swap_cell_storage();
  config();

  snn_t* network = create_network(info);
  network->layers[2].neurons[0].weights[0] = 5;

  write_snn(network);
  step(5);

  for (int i = 0; i < 32; i++) {
    char filename[20];
    sprintf(filename, "%02d.ps", i);
    step(1);
    readback();
    swap_cell_storage();
    read_states();
    matrix_t* state_matrix = get_states();
    print_matrix(state_matrix, info->state_bits);
    postscript_from_matrix_layer(state_matrix, 0, &colorfunction, filename);
    matrix_dispose(state_matrix);
  }
}

int main() {
  info = carp_connect();
  carp_reset();
  demo();
  carp_disconnect();
  return 0;
}
